# ---------------------------------------------------
# Netana - Graphs from data
# ---------------------------------------------------
# graphs of neighbours
# 21-24.9.2004 V. Batagelj

stand <-
# standardizes vector  x .
function(x){
  s <- sd(x)
  if (s > 0) (x - mean(x))/s  else  x - x
}

k.neighbor2Net <-
# stores network of first  k  neighbors for
# dissimilarity matrix  d  to file  fnet  in Pajek format.
#
# Example:
#   data(iris)
#   ir <- cbind(stand(iris[,1]),stand(iris[,2]),stand(iris[,3]),stand(iris[,4]))
#   k.neighbor2Net("iris5.net",as.matrix(dist(ir)),5)
function(fnet,d,k){
  net <- file(fnet,"w")
  n <- nrow(d); rn <- rownames(d)
  cat("*vertices",n,"\n",file=net)
  for (i in 1:n) cat(i," \"",rn[i],"\"\n",sep="",file=net)
  cat("*arcs\n",file=net)
  for (i in 1:n) for (j in order(d[i,])[1:k+1]) cat(i,j,d[i,j],"\n",file=net)
  close(net)
}

r.neighbor2Net <-
# stores network of  r-neighbors (d(v,u) <= r) for
# dissimilarity matrix  d  to file  fnet  in Pajek format.
#
# Example:
#   data(iris)
#   ir <- cbind(stand(iris[,1]),stand(iris[,2]),stand(iris[,3]),stand(iris[,4]))
#   r.neighbor2Net("irisR.net",as.matrix(dist(ir)),0.5)
function(fnet,d,r){
  net <- file(fnet,"w")
  n <- nrow(d); rn <- rownames(d)
  cat("*vertices",n,"\n",file=net)
  for (i in 1:n) cat(i," \"",rn[i],"\"\n",sep="",file=net)
  cat("*edges\n",file=net)
  for (i in 1:n){
    s <- order(d[i,]); j <- 1
    while (d[i,s[j]] <= r) {
      k <- s[j]; if (i < k) cat(i,k,d[i,k],"\n",file=net)
      j <- j+1
    }
  }
  close(net)
}

vector2Clu <-
# stores integer vector  v  as Pajek partition to file  fclu .
#
# Example:
#   data(iris)
#   vector2Clu("iris.clu",as.numeric(iris[,5]))
function(fclu,v){
  clu <- file(fclu,"w")
  n <- length(v)
  cat("*vertices",n,"\n",file=clu)
  for (i in 1:n) cat(v[i],"\n",file=clu)
  close(clu)
}

vector2Vec <-
# stores vector  v  as Pajek vector to file  fvec .
#
# Example:
#   data(iris)
#   vector2Vec("PetLen.vec",iris[,3])
function(fvec,v){
  vec <- file(fvec,"w")
  n <- length(v)
  cat("*vertices ",n,"\n",file=vec)
  for (i in 1:n) cat(v[i],"\n",file=vec)
  close(vec)
}

dist2net <-
# Pajek network (in *arcs/*edges format) on file  fnet
# transforms into network with dissimilarities  dist  between
# line endpoints as weights and store it to file  fdis .
#
# Example:
#   dist2net("iris5.net","irisDis.net",ir,dist)
#   dist2net("iris5.net","irisMan.net",ir,dist,method="manhattan")
function(fnet, fdis, dat, dist, ...){
  net <- file(fnet,"r"); dis <- file(fdis,"w"); copy <- TRUE
  while (copy) {
    t <- readLines(net,n=1)
    cat(t,"\n",file=dis)
    if (length(t)>0){
      if (substr(t,1,1) == "*") {
        c <- tolower(substr(t,2,2))
        copy <- (c != "e") & (c != "a")
        if (copy & (c != "v")) cat("Unexpected command in NET\n")
      }
    } else copy <- FALSE
  }
  copy <- TRUE
  digits <- c("0","1","2","3","4","5","6","7","8","9","-")
  while (copy) {
    t <- readLines(net,n=1)
    if (length(t)>0){
      if (substr(t,1,1) == "*") {
        c <- tolower(substr(t,2,2))
        copy <- (c == "e") | (c == "a")
        if (copy) cat(t,"\n",file=dis)
      } else {
        z <- unlist(strsplit(t," ")); z <- z[z != ""]
        u <- as.numeric(z[1]); v <- as.numeric(z[2])
        d <- dist(rbind(dat[u,],dat[v,]),...)
        j <- 3; if (is.element(substr(z[3],1,1),digits)) j <- 4
        cat(z[1],z[2],d,file=dis)
        if (j <= length(z)) for(i in j:length(z)) cat(z[i],file=dis)
        cat("\n",file=dis)
      }
    } else copy <- FALSE
  }
  close(dis); close(net)
}
